#!/bin/bash -eu
# Copyright (c) 2016 - 2020 DisplayLink (UK) Ltd.
# File autogenerated by udev-installer.sh script

DL_VENDOR=17e9
RETVAL=1

start_service()
{
  systemctl start --no-block displaylink-driver.service
}

stop_service()
{
  systemctl stop displaylink-driver.service
}

get_displaylink_dev_count()
{
   grep -qr "$DL_VENDOR" /sys/bus/usb/devices/*/idVendor
}

get_displaylink_symlink_count()
{
  root=$1
  count=0

  if [ ! -d "$root/displaylink/by-id" ]; then
    return 1
  fi

  while IFS= read -r -d '' file
  do
     test -c "$file" && (( count++ ))
  done <   <(find "$root"/displaylink/by-id -type l -print0 -exec realpath {} \; 2> /dev/null)
  [ "$count" -eq 0 ] && return 1
}

start_displaylink()
{
  get_displaylink_dev_count && start_service
}

stop_displaylink()
{
  root=$1
  get_displaylink_symlink_count "$root" || stop_service
}

remove_dldir_if_empty()
{
  root=$1
  (cd "$root"; rmdir -p --ignore-fail-on-non-empty displaylink/by-id)
}

create_displaylink_symlink()
{
  root=$1
  device_id=$2
  devnode=$3

  mkdir -p "$root"/displaylink/by-id
  ln -sf "$devnode" "$root"/displaylink/by-id/"$device_id"
}

unlink_displaylink_symlink()
{
   root=$1
   devname=$2

   dir="$root/displaylink/by-id"
   if [ -d "$dir" ]; then
     for f in "$dir"/*; do
       if [ ! -e "$f" ] || { [ -L "$f" ] && [ "$f" -ef "$devname" ]; }; then
         unlink "$f"
       fi
     done
     (cd "$root"; rmdir -p --ignore-fail-on-non-empty displaylink/by-id)
   fi
}

prune_broken_links()
{
  root=$1

  dir="$root/displaylink/by-id"
  if [ -d "$dir" ]; then
    find -L "$dir" -wholename "$dir" -o -type d -prune -o -type l -exec rm {} +
    remove_dldir_if_empty "$root"
  fi
}

disable_u1_u2()
{
    if [ -f "/sys$1/../port/usb3_lpm_permit" ]; then
      echo 0 > "/sys$1/../port/usb3_lpm_permit"
    else true
    fi
}

main()
{
  action=$1
  root=$2
  devpath=$3
  devnode=$5

  case "$action" in
    add)
      device_id=$4
      create_displaylink_symlink "$root" "$device_id" "$devnode"
      start_displaylink
      disable_u1_u2 "$devpath"
      ;;
    remove)
      devname=$3
      unlink_displaylink_symlink "$root" "$devname"
      stop_displaylink "$root"
      ;;
    START)
      start_displaylink
      ;;
    *)
      return 0
      ;;
  esac
}

ACTION="${ACTION:-}"

[ -n "$ACTION" ] || exit 0

if [ "$ACTION" = "add" ] && [ "$#" -ge 3 ]; then
  main "$ACTION" "$1" "$2" "$3" "$4"
  RETVAL="$?"
fi

if  [ "$ACTION" = "remove" ]; then
  if [ "$#" -ge 2 ]; then
    main "$ACTION" "$1" "$2"
    RETVAL="$?"
  else
    prune_broken_links "$1"
    RETVAL="$?"
  fi
fi

exit "$RETVAL"
